/*
 * otpCalc - A one time password calculator.
 *
 * Copyright (C) 2001 Anthony D. Urso
 * Copyright 2006-2021 Gentoo Authors
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "crypto.h"
#include "macros.h"
#include "utility.h"
#include "config.h"

void about(GtkWidget *widget, GtkWindow *parent)
{
	const gchar *authors[] = {
		"Anthony D. Urso",
		"Ulrich Müller <ulm@gentoo.org>",
		"Mart Raudsepp <leio@gentoo.org>",
		NULL
	};

	gtk_show_about_dialog (parent,
			       "version", VERSION,
			       "authors", authors,
			       "copyright", "Copyright (c) 2001, 2004 Anthony D. Urso\n"
					    "Copyright 2006-2021 Gentoo Authors",
			       "license-type", GTK_LICENSE_GPL_2_0,
			       "logo-icon-name", NULL,
			       "comments",
			       "Original homepage:\n"
			       "http://www.killa.net/infosec/otpCalc/\n\n"
			       "This version:\n"
			       "https://gitlab.com/ulm/otpcalc",
			       NULL);
}

void calculate(void)
{

	void (*hashes[5])();

	gchar *challenge, *passwd, *message, *response;
	struct tokens *set;
	extern GtkWidget *te_challenge, *te_passwd, *te_response;
	unsigned short i;
	unsigned char results[9];


	hashes[SKEY] = md5lite;
	hashes[MD4] = md4lite;
	hashes[MD5] = md5lite;
	hashes[RMD160] = rmd160lite;
	hashes[SHA1] = sha1lite;

	challenge = gtk_editable_get_chars(GTK_EDITABLE(te_challenge), 0, -1);
	set = g_malloc0(sizeof(struct tokens));
	set->seed = g_malloc(strlen(challenge));

	if (!parse(challenge, set)) {
		gtk_entry_set_text(GTK_ENTRY(te_response), "invalid challenge");
		g_free(challenge);
		g_free(set->seed);
		g_free(set);
		return;
	}

	g_free(challenge);

	passwd = gtk_editable_get_chars(GTK_EDITABLE(te_passwd), 0, -1);
	if (set->alg != SKEY) {
		gchar *tmp;

		tmp = g_utf8_strdown(set->seed, -1);
		g_free(set->seed);
		set->seed = tmp;
	}
	message = g_strconcat(set->seed, passwd, NULL);

	g_free(set->seed);
	g_free(passwd);

	(*hashes[set->alg])(message, strlen(message));
	memcpy(results, message, 8);
	g_free(message);

	for (i = 0; i < set->seq; i++)
		(*hashes[set->alg])(results, 8);

	g_free(set);

	results[8] = parity(results);

	response = g_malloc(31);
	sixwords(results, response);
	gtk_entry_set_text(GTK_ENTRY(te_response), response);
	g_free(response);
	gtk_editable_select_region(GTK_EDITABLE(te_response), 0, -1);

}

void clear(void)
{

	extern GtkWidget *te_challenge, *te_passwd, *te_response;

	gtk_editable_delete_text(GTK_EDITABLE(te_challenge), 0, -1);
	gtk_editable_delete_text(GTK_EDITABLE(te_passwd), 0, -1);
	gtk_editable_delete_text(GTK_EDITABLE(te_response), 0, -1);

}

void click(GtkWidget *unused, GtkWidget *button)
{

	gtk_button_clicked(GTK_BUTTON(button));

}

gboolean netwarn(void)
{
	int response;
	gchar *d;
	GtkWidget *di_netwarn, *la_netwarn;

	if (((d = getenv("DISPLAY")) == NULL) || g_str_has_prefix(d, ":"))
		return FALSE;

	di_netwarn = gtk_dialog_new_with_buttons("Warning!",
						 NULL,
						 0,
						 "_Yes", GTK_RESPONSE_YES,
						 "_No", GTK_RESPONSE_NO,
						 NULL);
	gtk_window_set_role(GTK_WINDOW(di_netwarn), "Warn");
	gtk_window_set_position(GTK_WINDOW(di_netwarn), GTK_WIN_POS_CENTER);

	la_netwarn = gtk_label_new(
		"It is unsafe to use otpCalc over an unencrypted X"
		" connection.\n\nProceed anyway?"
	);
	g_object_set(la_netwarn, "margin", 10, NULL);
	gtk_label_set_line_wrap(GTK_LABEL(la_netwarn), TRUE);
	gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(
					GTK_DIALOG(di_netwarn))),
			  la_netwarn);

	gtk_dialog_set_default_response(GTK_DIALOG(di_netwarn),
					GTK_RESPONSE_NO);

	gtk_widget_show_all(di_netwarn);

	response = gtk_dialog_run(GTK_DIALOG(di_netwarn));

	gtk_widget_destroy(di_netwarn);

	return response != GTK_RESPONSE_YES;
}


void selector(GtkWidget *widget, GtkSelectionData *selection, gpointer data)
{

	struct tokens *set;
	extern GtkWidget *te_challenge;
	unsigned short parsed;


	if (gtk_selection_data_get_length(selection) < 1)
		return;

	set = g_malloc(sizeof(struct tokens));
	set->seed = g_malloc(gtk_selection_data_get_length(selection));

	parsed = parse((const char *)gtk_selection_data_get_data(selection), set);

	g_free(set->seed);
	g_free(set);

	if (!parsed)
		return;

	gtk_entry_set_text(GTK_ENTRY(te_challenge),
			   (const gchar *)gtk_selection_data_get_data(selection));

}


void sethash(unsigned short new)
{

	extern unsigned short hash;

	hash = new;

}


gboolean focus(GtkWidget *widget, GdkEvent *event, gpointer data)
{

	static GdkAtom targets;
	extern unsigned short autopaste;


	if (!autopaste)
		return FALSE;

	targets = gdk_atom_intern("STRING", FALSE);

	gtk_selection_convert(widget, GDK_SELECTION_PRIMARY, targets,
			      GDK_CURRENT_TIME);

	return FALSE;
}
